#!/bin/bash
set -e

BZIP2_VERSION="1.0.8"
LIBMAD_VERSION="0.15.1b"
LIBJPEGTURBO_VERSION="3.1.0"
LIBPNG_VERSION="1.6.44"
LIBOGG_VERSION="1.3.5"
LIBVORBIS_VERSION="1.3.7"
FLAC_VERSION="1.4.3"
SDL2_VERSION="2.30.10"
BROTLI_VERSION="1.1.0"
HARFBUZZ_VERSION="10.1.0"
FREETYPE2_VERSION="2.13.3"
LIBDISCID_VERSION="0.6.4"
CJSON_VERSION="1.7.18"
ANCIENT_VERSION="2.2.0"
LIBICONV_VERSION="1.18"
GAMEMUSICEMU_VERSION="0.6.3"
UNIFONT_VERSION="16.0.02"
